@@ -59,6 +59,13 @@ void * ArrayListShift(ArrayList * list){
 	return ret;
 }
 
+void * ArrayListSet(ArrayList * list, srInt_64 n, void * new_elem){
+	if(list->numOfElements<=n) return NULL;
+	void * ret = list->elementList [n];
+	list->elementList [n] = new_elem;
+	return ret;
+}
+
 void * ArrayListPop(ArrayList * list){
 	if(list->numOfElements<1) return NULL;
 	return list->elementList [ -- list->numOfElements];
@@ -191,6 +198,10 @@ void ArrayListSort(ArrayList * list, int compare_L_minus_R(void * L_elem, void *
 		basic_sort(sortdata, list -> numOfElements, ArrayListSort_compare, ArrayListSort_exchange);
 }
 
+int ArrayListStringComparison(void * L_elem, void * R_elem){
+	return strcmp((char*)L_elem, (char*)R_elem);
+}
+
 int ArrayListLLUComparison(void * L_elem, void * R_elem){
 	srUInt_64 lint = L_elem-NULL;
 	srUInt_64 rint = R_elem-NULL;
@@ -284,6 +295,12 @@ ArrayList * HashTableKeys(HashTable * tab){
 	return ret;
 }
 
+srInt_64 HashTableInc(HashTable *tab, void *key){
+	void * oldv = HashTableGet(tab, key);
+	HashTablePut(tab, key, oldv+1);
+	return oldv - NULL+1;
+}
+
 HashTable *HashTableCreate(srInt_64 numOfBuckets) {
 	HashTable *hashTable;
 	int i;
