@@ -167,7 +167,7 @@ int write_result(propMapped_context * context)
 	{
 		FILE * outfp = f_subr_open(context -> output_file_name, "a");
 		#ifdef __MINGW32__
-		fprintf(outfp, "%s,%I64u,%I64u,%f\n", context -> input_file_name, context -> all_reads, context -> mapped_reads,  context -> mapped_reads*1./context -> all_reads);
+		fprintf(outfp, "%s,%" PRIu64 ",%" PRIu64 ",%f\n", context -> input_file_name, context -> all_reads, context -> mapped_reads,  context -> mapped_reads*1./context -> all_reads);
 		#else
 		fprintf(outfp, "%s,%llu,%llu,%f\n", context -> input_file_name, context -> all_reads, context -> mapped_reads,  context -> mapped_reads*1./context -> all_reads);
 		#endif
@@ -175,7 +175,7 @@ int write_result(propMapped_context * context)
 	}
 	char * objname = context -> is_fragments_counted? "fragment":"read";
 	#ifdef __MINGW32__
-	if(context -> verbose) SUBREADprintf("Finished. All records: %I64u; all %ss: %I64u; mapped %ss: %I64u; the mappability is %.2f%%\n", context->all_records, objname, context -> all_reads, objname, context -> mapped_reads, context -> mapped_reads*100./context -> all_reads);
+	if(context -> verbose) SUBREADprintf("Finished. All records: %" PRIu64 "; all %ss: %" PRIu64 "; mapped %ss: %" PRIu64 "; the mappability is %.2f%%\n", context->all_records, objname, context -> all_reads, objname, context -> mapped_reads, context -> mapped_reads*100./context -> all_reads);
 	#else
 	if(context -> verbose) SUBREADprintf("Finished. All records: %llu; all %ss: %llu; mapped %ss: %llu; the mappability is %.2f%%\n", context->all_records, objname, context -> all_reads, objname, context -> mapped_reads, context -> mapped_reads*100./context -> all_reads);
 	#endif
@@ -250,9 +250,9 @@ FILE * get_FP_by_read_name(propMapped_context * context, char * read_name)
 	{
 		char fname [MAX_FILE_NAME_LENGTH+40];
 		#ifdef __MINGW32__
-		sprintf(fname, "%s-%I64u.bin", context->temp_file_prefix, hash_key);
+		SUBreadSprintf(fname, MAX_FILE_NAME_LENGTH+40, "%s-%" PRIu64 ".bin", context->temp_file_prefix, hash_key);
 		#else
-		sprintf(fname, "%s-%llu.bin", context->temp_file_prefix, hash_key);
+		SUBreadSprintf(fname, MAX_FILE_NAME_LENGTH+40, "%s-%llu.bin", context->temp_file_prefix, hash_key);
 		#endif
 		ret = f_subr_open(fname, "wb");
 		HashTablePut(context -> split_fp_table, NULL+1+(int)(hash_key), ret);
@@ -315,7 +315,7 @@ int init_PE_sambam(propMapped_context * context)
 
 	if(context->temp_file_prefix[0]==0) strcpy(context->temp_file_prefix, "./");
 
-	sprintf(context->temp_file_prefix+strlen(context->temp_file_prefix), "/prpm-temp-sum-%06u-%s", getpid(), mac_rand);
+	SUBreadSprintf(context->temp_file_prefix+strlen(context->temp_file_prefix), 40, "/prpm-temp-sum-%06u-%s", getpid(), mac_rand);
 
 	_PROPMAPPED_delete_tmp_prefix = context -> temp_file_prefix;
 	signal (SIGTERM, PROPMAPPED_SIGINT_hook);
@@ -373,7 +373,7 @@ int prop_PE(propMapped_context * context)
 		HashTableSetHashFunction(rname_table, fc_chro_hash);
 		HashTableSetDeallocationFunctions(rname_table , free, NULL);
 
-		sprintf(fname, "%s-%d.bin", context->temp_file_prefix, bini);
+		SUBreadSprintf(fname, MAX_FILE_NAME_LENGTH+25, "%s-%d.bin", context->temp_file_prefix, bini);
 		FILE * fp = f_subr_open(fname, "rb");
 		if(fp)
 		{
