@@ -596,37 +596,37 @@ int convert_BAM_binary_to_SAM( SamBam_Reference_Info * chro_table, char * bam_bi
 	bin_len += 4;
 
 	int sam_ptr = 0, tmpint = 0;
-	sam_ptr += sprintf(sam_txt + sam_ptr, "%s\t", bam_bin+36);
+	sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%s\t", bam_bin+36);
 
 	memcpy(&tmpint, bam_bin + 16 ,4);
-	sam_ptr += sprintf(sam_txt + sam_ptr, "%d\t", (tmpint >> 16) & 0xffff);
+	sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%d\t", (tmpint >> 16) & 0xffff);
 	int cigar_opts = tmpint & 0xffff;
 
 	memcpy(&tmpint, bam_bin + 4  ,4);
 	int r1chro = tmpint;
-	sam_ptr += sprintf(sam_txt + sam_ptr, "%s\t", tmpint<0?"*":chro_table[tmpint].chro_name);
+	sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%s\t", tmpint<0?"*":chro_table[tmpint].chro_name);
 	memcpy(&tmpint, bam_bin + 8  ,4);
-	sam_ptr += sprintf(sam_txt + sam_ptr, "%d\t", tmpint+1);
+	sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%d\t", tmpint+1);
 	memcpy(&tmpint, bam_bin + 12 ,4);
-	sam_ptr += sprintf(sam_txt + sam_ptr, "%d\t", (tmpint >> 8) & 0xff);
+	sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%d\t", (tmpint >> 8) & 0xff);
 	int name_len = tmpint & 0xff;
 	int cigar_i;
 	for(cigar_i = 0; cigar_i < cigar_opts; cigar_i ++){
 		unsigned int cigarint=0;
 		memcpy(&cigarint, bam_bin + name_len + 36 + cigar_i * 4,4);
-		sam_ptr += sprintf(sam_txt + sam_ptr, "%u%c", cigarint >> 4, "MIDNSHP=X"[cigarint&0xf]);
+		sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%u%c", cigarint >> 4, "MIDNSHP=X"[cigarint&0xf]);
 	}
-	sam_ptr += sprintf(sam_txt + sam_ptr, "%s\t", cigar_i<1?"*":"");
+	sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%s\t", cigar_i<1?"*":"");
 	
 	memcpy(&tmpint, bam_bin + 24, 4);
 	//SUBREADprintf("CHRO_IDX=%d\n", tmpint);
-	sam_ptr += sprintf(sam_txt + sam_ptr, "%s\t", tmpint<0?"*":((tmpint == r1chro)?"=":chro_table[tmpint].chro_name));
+	sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%s\t", tmpint<0?"*":((tmpint == r1chro)?"=":chro_table[tmpint].chro_name));
 	
 	memcpy(&tmpint, bam_bin + 28, 4);
-	sam_ptr += sprintf(sam_txt + sam_ptr, "%d\t", tmpint+1);
+	sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%d\t", tmpint+1);
 
 	memcpy(&tmpint, bam_bin + 32, 4);
-	sam_ptr += sprintf(sam_txt + sam_ptr, "%d\t", tmpint);
+	sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%d\t", tmpint);
 
 	int seq_len;
 	memcpy(&seq_len, bam_bin + 20,4);
@@ -682,18 +682,18 @@ int convert_BAM_binary_to_SAM( SamBam_Reference_Info * chro_table, char * bam_bi
 					int tagval = 0;
 					memcpy(&tagval,  bam_bin + flex_ptr, type_bytes);
 					long long printv = is_signed?tagval:( (unsigned int) tagval );
-					if(elemtype == 'i') sam_ptr += sprintf(sam_txt + sam_ptr, "%d,", (int)printv);
-					if(elemtype == 'I') sam_ptr += sprintf(sam_txt + sam_ptr, "%u,", (unsigned int)printv);
+					if(elemtype == 'i') sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%d,", (int)printv);
+					if(elemtype == 'I') sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%u,", (unsigned int)printv);
 
-					if(elemtype == 's') sam_ptr += sprintf(sam_txt + sam_ptr, "%d,", (short)printv);
-					if(elemtype == 'S') sam_ptr += sprintf(sam_txt + sam_ptr, "%u,", (unsigned short)printv);
+					if(elemtype == 's') sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%d,", (short)printv);
+					if(elemtype == 'S') sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%u,", (unsigned short)printv);
 
-					if(elemtype == 'c') sam_ptr += sprintf(sam_txt + sam_ptr, "%d,", (char)printv);
-					if(elemtype == 'C') sam_ptr += sprintf(sam_txt + sam_ptr, "%u,", (unsigned char)printv);
+					if(elemtype == 'c') sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%d,", (char)printv);
+					if(elemtype == 'C') sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%u,", (unsigned char)printv);
 				}else{
 					float tagval = 0;
 					memcpy(&tagval,  bam_bin + flex_ptr, type_bytes);
-					sam_ptr += sprintf(sam_txt + sam_ptr, "%f,", tagval);
+					sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%f,", tagval);
 				}
 				flex_ptr += type_bytes;
 			}
@@ -734,9 +734,9 @@ int convert_BAM_binary_to_SAM( SamBam_Reference_Info * chro_table, char * bam_bi
 					memcpy(&tagval,  bam_bin + flex_ptr, type_bytes);
 					long long printv = is_signed?tagval:( (unsigned int) tagval );
 					#ifdef __MINGW32__
-					sam_ptr += sprintf(sam_txt + sam_ptr, "%I64d\t", printv);
+					sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%" PRId64 "\t", printv);
 					#else
-					sam_ptr += sprintf(sam_txt + sam_ptr, "%lld\t", printv);
+					sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%lld\t", printv);
 					#endif
 				}else if(is_string_type){
 					// type_bytes includes \0
@@ -749,7 +749,7 @@ int convert_BAM_binary_to_SAM( SamBam_Reference_Info * chro_table, char * bam_bi
 				}else if(is_float_type){
 					float tagval = 0;
 					memcpy(&tagval,  bam_bin + flex_ptr, type_bytes);
-					sam_ptr += sprintf(sam_txt + sam_ptr, "%f\t", tagval);
+					sam_ptr += SUBreadSprintf(sam_txt + sam_ptr, 50000, "%f\t", tagval);
 				}else if(is_char_type){
 					sam_txt[ sam_ptr++ ] = bam_bin[flex_ptr];
 					sam_txt[ sam_ptr++ ] = '\t';
@@ -832,7 +832,7 @@ int PBam_chunk_gets(char * chunk, int *chunk_ptr, int chunk_limit, SamBam_Refere
 		memcpy(&cigar_piece,  chunk+(*chunk_ptr),4);
 		(*chunk_ptr)+=4;
 
-		sprintf(cigar_piece_buf, "%u%c", cigar_piece>>4, cigar_op_char(cigar_piece&0xf));
+		SUBreadSprintf(cigar_piece_buf, 12, "%u%c", cigar_piece>>4, cigar_op_char(cigar_piece&0xf));
 		if(strlen(cigar_piece_buf)+strlen(aln->cigar)<BAM_MAX_CIGAR_LEN-1)
 			strcat(aln->cigar, cigar_piece_buf);
 		else
@@ -901,12 +901,12 @@ int PBam_chunk_gets(char * chunk, int *chunk_ptr, int chunk_limit, SamBam_Refere
 				int tmpi = 0;
 				memcpy(&tmpi, chunk+(*chunk_ptr),delta);
 				if(tmpi >= 0 && extra_len < CORE_ADDITIONAL_INFO_LENGTH - 18){
-					int sret = sprintf(extra_tags + strlen(extra_tags), "\t%c%c:i:%d", extag[0], extag[1], tmpi);
+					int sret = SUBreadSprintf(extra_tags + strlen(extra_tags), 18, "\t%c%c:i:%d", extag[0], extag[1], tmpi);
 					extra_len += sret;
 				}
 			}else if(extype == 'Z'){
 				if(extra_len < CORE_ADDITIONAL_INFO_LENGTH - 7 - delta){
-					sprintf(extra_tags + strlen(extra_tags), "\t%c%c:Z:", extag[0], extag[1]);
+					SUBreadSprintf(extra_tags + strlen(extra_tags), 10, "\t%c%c:Z:", extag[0], extag[1]);
 					extra_len += 6;
 					*(extra_tags + strlen(extra_tags)+delta-1) = 0;
 					memcpy(extra_tags + strlen(extra_tags), chunk + (*chunk_ptr), delta - 1);
@@ -914,7 +914,7 @@ int PBam_chunk_gets(char * chunk, int *chunk_ptr, int chunk_limit, SamBam_Refere
 				}
 			}else if(extype == 'A'){
 				if(extra_len < CORE_ADDITIONAL_INFO_LENGTH - 8){
-					int sret = sprintf(extra_tags + strlen(extra_tags), "\t%c%c:A:%c", extag[0], extag[1], *(chunk + *chunk_ptr) );
+					int sret = SUBreadSprintf(extra_tags + strlen(extra_tags), 10, "\t%c%c:A:%c", extag[0], extag[1], *(chunk + *chunk_ptr) );
 					extra_len += sret;
 				}
 			}
@@ -988,7 +988,7 @@ int PBam_chunk_gets(char * chunk, int *chunk_ptr, int chunk_limit, SamBam_Refere
 	//fprintf(stderr, "HN_TAG=%d\n", nh_val	);
 
 	#ifdef __MINGW32__
-	int plen = snprintf(buff, buff_len-1, "%s\t%u\t%s\t%u\t%d\t%s\t%s\t%u\t%I64d\t%s\t%s%s\n%c", aln -> read_name, aln -> flags , chro_name, chro_offset, aln -> mapping_quality, cigar, mate_chro_name, mate_chro_offset, templete_length, aln -> sequence , aln -> seq_quality, extra_tags, 0);
+	int plen = snprintf(buff, buff_len-1, "%s\t%u\t%s\t%u\t%d\t%s\t%s\t%u\t%" PRId64 "\t%s\t%s%s\n%c", aln -> read_name, aln -> flags , chro_name, chro_offset, aln -> mapping_quality, cigar, mate_chro_name, mate_chro_offset, templete_length, aln -> sequence , aln -> seq_quality, extra_tags, 0);
 	#else
 	int plen = snprintf(buff, buff_len-1, "%s\t%u\t%s\t%u\t%d\t%s\t%s\t%u\t%lld\t%s\t%s%s\n%c", aln -> read_name, aln -> flags , chro_name, chro_offset, aln -> mapping_quality, cigar, mate_chro_name, mate_chro_offset, templete_length, aln -> sequence , aln -> seq_quality, extra_tags, 0);
 	#endif
@@ -1103,7 +1103,7 @@ int SamBam_writer_create(SamBam_Writer * writer, char * BAM_fname, int threads,
 		writer -> bam_fp = f_subr_open(BAM_fname, "wb");
 		if(sort_reads_by_coord){
 			char tname[MAX_FILE_NAME_LENGTH];
-			sprintf(tname, "%s.bai", BAM_fname);
+			SUBreadSprintf(tname, MAX_FILE_NAME_LENGTH, "%s.bai", BAM_fname);
 			writer -> BAI_fp = f_subr_open(tname, "wb");
 			worker_master_mutex_init(&writer->sorted_notifier, threads);
 		}
@@ -1877,7 +1877,7 @@ int SamBam_writer_sort_buff_one_write(SamBam_Writer * writer, char * bin, int bi
 
 	char tmpfname[MAX_FILE_NAME_LENGTH+40];
 	if(writer -> threads>1) subread_lock_occupy(&writer -> thread_bam_lock);
-	sprintf(tmpfname, "%s-%06d.sortedbin", writer -> tmpf_prefix, writer -> sorted_batch_id++);
+	SUBreadSprintf(tmpfname, MAX_FILE_NAME_LENGTH+40, "%s-%06d.sortedbin", writer -> tmpf_prefix, writer -> sorted_batch_id++);
 	if(writer -> threads>1) subread_lock_release(&writer -> thread_bam_lock);
 	FILE * tofp  = fopen(tmpfname, "wb");
 	if(tofp){
@@ -1911,15 +1911,14 @@ unsigned long long SamBam_writer_sort_bins_to_BAM_FP_pos(FILE * fp){
 
 
 int SamBam_writer_calc_cigar_span(char * bin){
-	int cops = 0, rname_len = 0, ret = 0;
-	memcpy(&cops, bin+12, 4);
-	memcpy(&rname_len, bin+8, 4);
-	rname_len = rname_len & 0xff;
-	cops = cops & 0xffff;
+	int cops, rname_len, ret = 0;
+	cops = *(short *)(bin+12);
+	rname_len = bin[8];
 	int ii;
+	char * binbase = bin+32+rname_len;
 	for(ii = 0; ii < cops ; ii++){
 		unsigned int copt = 0;
-		memcpy(&copt, bin+32+rname_len+4*ii, 4);
+		copt = *(int*)(binbase+4*ii);
 		int copt_char = copt & 0xf;
 		unsigned int copt_len = copt >> 4;
 		if(copt_char == 0 || copt_char == 2 || copt_char == 3 || copt_char == 7 || copt_char == 8) ret += copt_len;
@@ -1939,7 +1938,6 @@ void SamBam_writer_sort_bins_to_BAM_test_bins(SamBam_Writer * writer, HashTable
 	binno = bin_mq_nl>>16;
 
 	int cigar_span = SamBam_writer_calc_cigar_span(writer -> chunk_buffer + inbin_pos);
-
 	int this_w16_no = (pos + cigar_span) >>14;	// WIN is calculated on 0-based pos.
 	unsigned long long this_Vpos = writer -> this_bam_block_no<<16 | (inbin_pos-4);
 
@@ -2091,7 +2089,7 @@ void SamBam_writer_one_thread_merge_sortedbins(SamBam_Writer * writer){
 			if(this_size < 2) break;
 
 			//SUBREADprintf("CREATE_MERGE %d\n", new_bins + writer -> sorted_batch_id);
-			sprintf(tfp , "%s-%06d.sortedbin", writer -> tmpf_prefix, writer -> sorted_batch_id + (new_bins++) );
+			SUBreadSprintf(tfp, MAX_FILE_NAME_LENGTH+50 , "%s-%06d.sortedbin", writer -> tmpf_prefix, writer -> sorted_batch_id + (new_bins++) );
 			FILE * outbinfp = fopen(tfp,"wb");
 
 			FILE ** sb_fps = malloc(sizeof(FILE *) * this_size);
@@ -2103,7 +2101,7 @@ void SamBam_writer_one_thread_merge_sortedbins(SamBam_Writer * writer){
 				char tfpx[MAX_FILE_NAME_LENGTH+50];
 				current_min_fps[bii] = SUBREAD_MAX_ULONGLONG;
 
-				sprintf(tfpx , "%s-%06d.sortedbin", writer -> tmpf_prefix, bii + merge_i);
+				SUBreadSprintf(tfpx , MAX_FILE_NAME_LENGTH+50, "%s-%06d.sortedbin", writer -> tmpf_prefix, bii + merge_i);
 				sb_fps[bii] = fopen(tfpx,"rb");
 				current_min_fps[bii] = SamBam_writer_sort_bins_to_BAM_FP_pos(sb_fps[bii]);
 				if(current_min_fps[bii] < SUBREAD_MAX_ULONGLONG && current_min_fps[bii] < current_min){
@@ -2140,7 +2138,7 @@ void SamBam_writer_one_thread_merge_sortedbins(SamBam_Writer * writer){
 
 			for(bii = 0; bii < this_size; bii++){
 				fclose(sb_fps[bii]);
-				sprintf(tfp , "%s-%06d.sortedbin", writer -> tmpf_prefix, merge_i + bii);
+				SUBreadSprintf(tfp, MAX_FILE_NAME_LENGTH+50 , "%s-%06d.sortedbin", writer -> tmpf_prefix, merge_i + bii);
 				//SUBREADprintf("    DEL_MERGE %d\n", bii);
 				unlink(tfp);
 			}
@@ -2361,7 +2359,7 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 		char tfp[MAX_FILE_NAME_LENGTH+40];
 		current_min_fps[bii] = SUBREAD_MAX_ULONGLONG;
 
-		sprintf(tfp , "%s-%06d.sortedbin", writer -> tmpf_prefix, bii);
+		SUBreadSprintf(tfp, MAX_FILE_NAME_LENGTH+40, "%s-%06d.sortedbin", writer -> tmpf_prefix, bii);
 		sb_fps[bii] = fopen(tfp,"rb");
 		if(sb_fps[bii]!=NULL){
 			current_min_fps[bii] = SamBam_writer_sort_bins_to_BAM_FP_pos(sb_fps[bii]);
@@ -2378,7 +2376,7 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 	SAMBAM_renew_BAItabs;
 
 	int chro_no = (int)(current_min >> 32);
-	int old_chro_no = -1, last_written_BAI_chro = -1;;
+	int old_chro_no = -1;
 
 	int wlen = fwrite("BAI\1", 4, 1, writer -> BAI_fp);
 	if(wlen < 1) assert(wlen >0);
@@ -2413,7 +2411,6 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 		if(old_chro_no >=0 && chro_no != old_chro_no){ // if there is old_chro && if has to write. Note: old_chro_no<0 means we reached the "unmapped" part of the sorted temp files
 			//SUBREADprintf("\n====== W1CHR %d =====\n", old_chro_no);
 			SamBam_write_BAI_for_1chr(writer, &bin_chunks_table, &bins_list, &win16k_list);
-			last_written_BAI_chro = old_chro_no;
 			SAMBAM_write_empty_chros(old_chro_no +1, chro_no < 0?n_ref:chro_no);
 			SAMBAM_renew_BAItabs;
 			SAMBAM_reset_sorting_writer;
@@ -2444,7 +2441,7 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 		if(!sb_fps[bii]) continue;
 
 		char tfp[MAX_FILE_NAME_LENGTH+40];
-		sprintf(tfp , "%s-%06d.sortedbin", writer -> tmpf_prefix, bii);
+		SUBreadSprintf(tfp, MAX_FILE_NAME_LENGTH+40, "%s-%06d.sortedbin", writer -> tmpf_prefix, bii);
 		fclose(sb_fps[bii]);
 		unlink(tfp);
 	}
@@ -2497,3 +2494,181 @@ void SamBam_writer_finalise_one_thread(SamBam_Writer * writer){
 		}
 	}
 }
+
+
+unsigned int FC_CRC32(char * dat, int len){
+	unsigned int crc0 = crc32(0, NULL, 0);
+	unsigned int ret = crc32(crc0, (unsigned char *)dat, len);
+	return ret;
+}
+
+void simple_bam_writer_deallocate_index_per_chro(void * p){
+	struct simple_bam_writer_index_per_chro * ch = p;
+	HashTableDestroy(ch->index_binP1_table);
+	ArrayListDestroy(ch->index_binP0_list);
+	ArrayListDestroy(ch->win16k_list);
+	free(ch);
+}
+
+
+
+struct simple_bam_writer_index_per_chro * simple_bam_writer_new_index_per_chro(){
+	struct simple_bam_writer_index_per_chro * ret = malloc(sizeof(struct simple_bam_writer_index_per_chro ));
+	ret -> index_binP1_table = HashTableCreate(4000);
+	HashTableSetDeallocationFunctions(ret -> index_binP1_table, NULL, (void (*) (void*)) ArrayListDestroy);
+	ret -> index_binP0_list = ArrayListCreate(20000);
+	ret -> win16k_list = ArrayListCreate(20000);
+	return ret;
+}
+
+void simple_bam_writer_update_index(simple_bam_writer * writer, char * rbin, int binlen, srInt_64 block_number, int inbin_pos){
+	int chro_no=0;
+	memcpy(&chro_no, rbin + 4, 4);
+	if(chro_no<0)return;
+
+	unsigned int pos, bin_mq_nl=0;
+	pos = *(unsigned int*)(rbin + 8);
+	bin_mq_nl = *(unsigned int*)(rbin + 12);
+
+	struct simple_bam_writer_index_per_chro * index_chro = HashTableGet(writer -> index_per_chro, NULL+chro_no+1);
+	if(NULL==index_chro){
+		index_chro = simple_bam_writer_new_index_per_chro();
+		HashTablePut(writer -> index_per_chro, NULL+chro_no+1, index_chro);
+	}
+
+	unsigned int binno = bin_mq_nl>>16;
+	int cigar_span = SamBam_writer_calc_cigar_span(rbin +4);
+	int this_w16_no = (pos + cigar_span) >>14;      // WIN is calculated on 0-based pos.
+	unsigned long long this_Vpos = block_number<<16 | inbin_pos;
+	ArrayList * win16k_list = index_chro -> win16k_list;
+	// if this read is after the maximum coordinate in the win16k list: all elements before last one and this one starts at this read.
+	if(this_w16_no > win16k_list->numOfElements){
+		int bbi;
+		for(bbi = win16k_list->numOfElements; bbi <=this_w16_no; bbi++)
+			ArrayListPush(win16k_list, NULL+ this_Vpos);
+	}
+
+	ArrayList * this_bin_chunks = HashTableGet(index_chro -> index_binP1_table, NULL+binno+1);
+	if(NULL == this_bin_chunks){
+		this_bin_chunks = ArrayListCreate(4);
+		HashTablePut(index_chro -> index_binP1_table, NULL+binno+1, this_bin_chunks);
+		ArrayListPush(index_chro -> index_binP0_list, NULL+binno);
+	}
+	int found = 0;
+	// a bin is not necessarily continuous. Say, a top-level bin only contains a few reads (most reads a in low-level bins), but their locations are everywhere
+
+	if(this_bin_chunks -> numOfElements > 0){
+		long long diff = this_Vpos >>16;
+		diff -=(this_bin_chunks -> elementList [ this_bin_chunks -> numOfElements - 1] - NULL)>>16;
+		if(diff < MAX_ALLOWED_GAP_IN_BAI_CHUNK){
+			this_bin_chunks -> elementList [ this_bin_chunks -> numOfElements - 1] = NULL+this_Vpos + binlen + 4;
+			found = 1;
+		}
+	}
+	// if the last chunk in this bin isn't good to be extended (too far from the file location of the new read), a new chunk is created.
+	if(!found){
+		ArrayListPush(this_bin_chunks, NULL + this_Vpos);
+		ArrayListPush(this_bin_chunks, NULL + this_Vpos + binlen+4);
+	}
+}
+
+void simple_bam_write_compressed_block(simple_bam_writer * writer,char *obuf, int olen, int ilen, unsigned int crcval, srInt_64 block_number){
+	if(block_number >= 0)HashTablePut(writer -> bam_blockP1_to_offset0B_table, NULL+1+block_number, NULL+ftello(writer -> bam_FP));
+	fwrite("\x1f\x8b\x8\x4\0\0\0\0\0\0\x6\0", 1, 12, writer -> bam_FP);
+	fwrite("\x42\x43\x2\0", 1, 4, writer -> bam_FP);
+
+	int BSIZE = olen+19+6;
+	fwrite(&BSIZE, 1, 2, writer -> bam_FP);
+	fwrite(obuf, 1, olen, writer -> bam_FP);
+	fwrite(&crcval, 1, 4, writer -> bam_FP);
+	fwrite(&ilen, 1, 4, writer -> bam_FP);
+}
+
+void simple_bam_write(void * bin, int binlen, simple_bam_writer * writer, int force_flush){
+	while(binlen > 0 || (force_flush && writer->inbin_len)){
+		int concatinate_binlen = min(binlen, 63000 - writer->inbin_len);
+		memcpy(writer->inbin+writer->inbin_len, bin, concatinate_binlen);
+
+		writer->inbin_len += concatinate_binlen;
+		bin += concatinate_binlen;
+		binlen -= concatinate_binlen;
+		if(writer->inbin_len >=63000 || force_flush){
+			deflateInit2(&writer -> strm, Z_BEST_SPEED, Z_DEFLATED, -15, Z_DEFAULT_MEM_LEVEL, Z_DEFAULT_STRATEGY);
+			char obuf[MERGER_WORKER_BINSIZE];
+			writer->strm.next_in = (unsigned char *)writer->inbin;
+			writer->strm.avail_in = writer->inbin_len;
+			writer->strm.next_out = (unsigned char *)obuf;
+			writer->strm.avail_out = MERGER_WORKER_BINSIZE;
+			deflate(&writer->strm, Z_FINISH);
+			int have = MERGER_WORKER_BINSIZE-writer->strm.avail_out;
+			simple_bam_write_compressed_block(writer, obuf, have, writer->inbin_len, FC_CRC32(writer->inbin, writer->inbin_len), -1);
+			writer->inbin_len=0;
+			deflateEnd(&writer -> strm);
+		}
+	}
+}
+
+simple_bam_writer * simple_bam_create(char * fname){
+	simple_bam_writer * ret = calloc(sizeof(simple_bam_writer), 1);
+	ret -> bam_FP = fopen(fname, "wb");
+	ret -> bam_blockP1_to_offset0B_table = HashTableCreate(100000);
+	simple_bam_write("BAM\1", 4, ret, 0);
+
+	char bainame [strlen(fname)+10];
+	strcpy(bainame , fname);
+	strcat(bainame, ".bai");
+	ret -> bai_FP = fopen(bainame, "wb");
+	fwrite("BAI\1", 1, 4, ret -> bai_FP);
+	ret -> index_per_chro = HashTableCreate(1000);
+	HashTableSetDeallocationFunctions(ret -> index_per_chro , NULL , simple_bam_writer_deallocate_index_per_chro);
+	return ret;
+}
+
+#define vpos_to_rpos rposone = (vposone & 0xffff ) + ( (HashTableGet(writer -> bam_blockP1_to_offset0B_table, NULL+1+(vposone >>16)) - NULL) << 16 )
+
+#define BAM_EOF_MARKER "\x1f\x8b\x08\x04\0\0\0\0\0\xff\x06\0\x42\x43\x02\0\x1b\0\x03\0\0\0\0\0\0\0\0\0"
+void simple_bam_close(simple_bam_writer * writer){
+	fwrite(BAM_EOF_MARKER, 1, 28, writer -> bam_FP);
+	fclose(writer -> bam_FP);
+
+	fwrite(&writer -> total_chromosomes, 1, 4, writer -> bai_FP);
+	int chri;
+	for(chri=0; chri<writer -> total_chromosomes; chri++){
+		struct simple_bam_writer_index_per_chro *this_idx = HashTableGet(writer -> index_per_chro , NULL+1+chri);
+		if(NULL == this_idx ){
+			fwrite("\0\0\0\0\0\0\0\0", 1, 8, writer -> bai_FP);//0 intervals and 0 bins
+		}else{
+			HashTable * new_tab=NULL;
+			ArrayList * new_arr=NULL;
+			SamBam_writer_optimize_bins(this_idx -> index_binP1_table, this_idx -> index_binP0_list ,& new_tab, & new_arr);
+			this_idx -> index_binP1_table = new_tab;
+			this_idx -> index_binP0_list = new_arr;
+			fwrite(&this_idx -> index_binP0_list->numOfElements ,1, 4, writer -> bai_FP);
+			int bini;
+			for(bini = 0; bini < this_idx -> index_binP0_list -> numOfElements; bini ++){
+				int binno = ArrayListGet(this_idx -> index_binP0_list, bini)-NULL;
+				ArrayList * bingaps = HashTableGet(this_idx -> index_binP1_table, NULL+1+binno);
+				srInt_64 gapi = bingaps -> numOfElements/2;
+				fwrite(&binno, 1, 4, writer -> bai_FP);
+				fwrite(&gapi ,1, 4, writer -> bai_FP);
+				for(gapi = 0; gapi < bingaps -> numOfElements; gapi++){
+					srInt_64 rposone, vposone = ArrayListGet(bingaps , gapi)-NULL;
+					vpos_to_rpos;
+					fwrite(&rposone, 1, 8, writer -> bai_FP);
+				}
+			}
+
+			fwrite(&this_idx -> win16k_list -> numOfElements ,1, 4, writer -> bai_FP);
+			for(bini = 0; bini < this_idx -> win16k_list -> numOfElements; bini ++){
+				srInt_64 rposone, vposone = ArrayListGet(this_idx -> win16k_list , bini )-NULL;
+				vpos_to_rpos;
+				fwrite(&rposone, 1, 8, writer -> bai_FP);
+			}
+		}
+	}
+	HashTableDestroy(writer -> index_per_chro);
+	fclose(writer -> bai_FP);
+	free(writer);
+}
+
+
